@@ -1,41 +1,6 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 Kurt Pattyn <pattyn.kurt@gmail.com>.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtWebSockets module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 Kurt Pattyn <pattyn.kurt@gmail.com>.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:critical reason:network-protocol
 /*!
     \class QWebSocketDataProcessor
     The class QWebSocketDataProcessor is responsible for reading, validating and
@@ -80,16 +45,14 @@ QWebSocketDataProcessor::QWebSocketDataProcessor(QObject *parent) :
     m_isControlFrame(false),
     m_hasMask(false),
     m_mask(0),
-    m_binaryMessage(),
-    m_textMessage(),
     m_payloadLength(0),
-    m_decoder(QStringDecoder(QStringDecoder::Utf8, QStringDecoder::Flag::Stateless
-        | QStringDecoder::Flag::ConvertInvalidToNull)),
-    m_waitTimer(new QTimer(this))
+    m_decoder(QStringDecoder(QStringDecoder::Utf8, QStringDecoder::Flag::ConvertInvalidToNull)),
+    m_waitTimer(new QChronoTimer(this))
 {
     clear();
     // initialize the internal timeout timer
-    m_waitTimer->setInterval(5000);
+    using namespace std::chrono_literals;
+    m_waitTimer->setInterval(5s);
     m_waitTimer->setSingleShot(true);
     m_waitTimer->callOnTimeout(this, &QWebSocketDataProcessor::timeout);
 }
@@ -145,6 +108,24 @@ quint64 QWebSocketDataProcessor::maxFrameSize()
    return QWebSocketFrame::maxFrameSize();
 }
 
+/*!
+    \internal
+*/
+void QWebSocketDataProcessor::setIdleTimeout(std::chrono::milliseconds timeout)
+{
+    Q_ASSERT(!m_waitTimer->isActive());
+    m_waitTimer->setInterval(timeout);
+}
+
+/*!
+    \internal
+*/
+std::chrono::milliseconds QWebSocketDataProcessor::idleTimeout() const
+{
+    using namespace std::chrono;
+    return duration_cast<milliseconds>(m_waitTimer->interval());
+}
+
 /*!
     \internal
 
@@ -160,7 +141,7 @@ bool QWebSocketDataProcessor::process(QIODevice *pIoDevice)
         if (!frame.isDone()) {
             // waiting for more data available
             QObject::connect(pIoDevice, &QIODevice::readyRead,
-                             m_waitTimer, &QTimer::stop, Qt::UniqueConnection);
+                             m_waitTimer, &QChronoTimer::stop, Qt::UniqueConnection);
             m_waitTimer->start();
             return false;
         } else if (Q_LIKELY(frame.isValid())) {
@@ -188,9 +169,9 @@ bool QWebSocketDataProcessor::process(QIODevice *pIoDevice)
                     m_isFragmented = !frame.isFinalFrame();
                 }
                 quint64 messageLength = m_opCode == QWebSocketProtocol::OpCodeText
-                        ? quint64(m_textMessage.length())
-                        : quint64(m_binaryMessage.length());
-                if (Q_UNLIKELY((messageLength + quint64(frame.payload().length())) >
+                        ? quint64(m_textMessage.size())
+                        : quint64(m_binaryMessage.size());
+                if (Q_UNLIKELY((messageLength + quint64(frame.payload().size())) >
                                maxAllowedMessageSize())) {
                     clear();
                     Q_EMIT errorEncountered(QWebSocketProtocol::CloseCodeTooMuchData,
@@ -201,6 +182,18 @@ bool QWebSocketDataProcessor::process(QIODevice *pIoDevice)
                 bool isFinalFrame = frame.isFinalFrame();
                 if (m_opCode == QWebSocketProtocol::OpCodeText) {
                     QString frameTxt = m_decoder(frame.payload());
+                    if (frame.isFinalFrame()) {
+                        // ### We lack API on the QStringDecoder to check if there is an incomplete
+                        // sequence inside its state object. But we need to make sure we are not
+                        // passing on an empty, or potentially corrupted message to our users.
+                        // By forcing the decoder to process a 0-byte we force it to evaluate the
+                        // current state as the full sequence, making it process it as an error if
+                        // it's not complete.
+                        frameTxt += m_decoder(QByteArrayView("\0", 1));
+                        // As a downside, we then always have to remove the 0-byte that we just
+                        // added:
+                        frameTxt.chop(1);
+                    }
                     if (Q_UNLIKELY(m_decoder.hasError())) {
                         clear();
                         Q_EMIT errorEncountered(QWebSocketProtocol::CloseCodeWrongDatatype,
